package com.example.ellegadodepintia.minijuego2

import android.content.Intent
import android.graphics.drawable.AnimationDrawable
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.LayoutInflater
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import android.view.animation.AnimationUtils
import android.widget.Button
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.DialogFragment
import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.soundManager.SoundManager
import kotlin.random.Random

class Minijuego2 : AppCompatActivity() {

    private lateinit var botonIniciar: ImageView
    private lateinit var protagonista: ImageView
    private lateinit var layout: RelativeLayout
    private lateinit var titulo: ImageView
    private lateinit var descripcion: ImageView
    private lateinit var puntuacionTextView: TextView
    private lateinit var botonReinicio: ImageView
    private lateinit var fondo1: ImageView
    private lateinit var fondo2: ImageView
    private lateinit var nube: ImageView
    private lateinit var soundManager: SoundManager

    private var puntuacion = 0
    private var gravedad = 1f
    private var velObs = 10f
    private var aceleracionSubida = -1f
    private var velocidadMaximaCaida = 10f
    private var velocidadMaximaSubida = -15f
    private var velocidad = 0f
    private var jugando = false
    private var pulsacion = false
    private val obstaculos = mutableListOf<ImageView>()
    private val handler = Handler(Looper.getMainLooper())
    private var obstacleTimer: Runnable? = null
    private var velocidadFondo = 5f

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_minijuego2)
        inicializarComponentes()
        configurarAnimaciones()
        configurarBotonIniciar()
        configurarListenerPantalla()
    }

    private fun inicializarComponentes() {
        soundManager = SoundManager(this)
        botonIniciar = findViewById(R.id.boton_iniciar)
        protagonista = findViewById(R.id.protagonista)
        layout = findViewById(R.id.layout)
        titulo = findViewById(R.id.titulo)
        descripcion = findViewById(R.id.descripcion)
        puntuacionTextView = findViewById(R.id.puntuacion)
        botonReinicio = findViewById(R.id.boton_reinicio)
        nube = findViewById(R.id.nube)
        fondo1 = findViewById(R.id.fondo1)
        fondo2 = findViewById(R.id.fondo2)
        val tutorialButton = findViewById<ImageView>(R.id.descripcion)
        tutorialButton.setOnClickListener {
            // Mostrar el modal
            val modal = ModalFragment()
            modal.show(supportFragmentManager, "ModalFragment")
        }

    }

    private fun configurarAnimaciones() {
        val imagenProtagonista = findViewById<ImageView>(R.id.protagonista)
        val idleAnimation = imagenProtagonista.drawable as? AnimationDrawable
        idleAnimation?.start()

        val animacion = AnimationUtils.loadAnimation(this, R.anim.animatitu)
        titulo.startAnimation(animacion)
    }

    private fun configurarBotonIniciar() {
        botonIniciar.setOnClickListener {
            soundManager.playSound(R.raw.sound_button,100)
            iniciarJuego()
            protagonista.y = (layout.height / 2).toFloat()
            fondo2.x = fondo1.x + fondo1.width * 4
            protagonista.x = protagonista.x + (layout.width*0.1).toInt()
            nube.x = protagonista.x + (layout.width*0.03).toInt()
        }
    }

    private fun configurarListenerPantalla() {
        findViewById<View>(android.R.id.content).setOnTouchListener { view, event ->
            when (event.action) {
                MotionEvent.ACTION_DOWN -> {
                    // Al tocar la pantalla, el personaje sube
                    velocidad = velocidadMaximaSubida // Impulsa al protagonista hacia arriba
                }
            }
            true
        }
    }


    private fun iniciarJuego() {
        soundManager.playSoundLoop(R.raw.sound_nubes,100)
        ocultarElementosInicioYMostrarProtagonista()
        inicializarJuego()
        iniciarBucleJuego()
        iniciarTemporizadorGenerarObstaculos()
    }

    private fun ocultarElementosInicioYMostrarProtagonista() {
        titulo.imageAlpha = 0
        titulo.clearAnimation()
        descripcion.visibility = View.GONE
        botonIniciar.visibility = View.GONE
        protagonista.visibility = View.VISIBLE
        nube.visibility = View.VISIBLE
        puntuacionTextView.visibility = View.VISIBLE
        protagonista.setElevation(10f)
        nube.setElevation(10f)
        botonReinicio.setElevation(10f)
        puntuacionTextView.setElevation(10f)
    }

    private fun inicializarJuego() {
        jugando = true
        velocidad = 0f // Inicializar la velocidad en 0 al inicio
        obstaculos.clear() // Limpiar obstáculos anteriores
        puntuacion = 0
    }

    private fun iniciarBucleJuego() {
        handler.post(object : Runnable {
            override fun run() {
                if (jugando) {
                    actualizarPosicionProtagonista()
                    moverObstaculos()
                    verificarColisiones()
                    moverFondo()
                    handler.postDelayed(this, 16)
                }
            }
        })
    }

    private fun iniciarTemporizadorGenerarObstaculos() {
        obstacleTimer = object : Runnable {
            override fun run() {
                if (jugando) {
                    generarObstaculo()
                    val delay = if (puntuacion > 9) {
                        1100L
                    } else {
                        (2000 - puntuacion * 100).toLong()
                    }
                    handler.postDelayed(this, delay)
                }
            }
        }
        handler.post(obstacleTimer!!)
    }

    private fun actualizarPosicionProtagonista() {
        val altitud = protagonista.y
        velocidad = if (pulsacion) {
            (velocidad + aceleracionSubida).coerceAtLeast(velocidadMaximaSubida)
        } else {
            (velocidad + gravedad).coerceAtMost(velocidadMaximaCaida)
        }
        val nuevaPosicionY = altitud + velocidad
        val techoPantalla = findViewById<View>(android.R.id.content).height
        if (nuevaPosicionY < 10) {
            protagonista.y = 10f
            velocidad = 0f
        } else if (nuevaPosicionY + protagonista.height < techoPantalla) {
            protagonista.y = nuevaPosicionY
            nube.y = protagonista.y  + (layout.height*0.05).toInt()
        }
    }

    private fun generarObstaculo() {
        val anchuraPantalla = layout.width
        val alturaPantalla = layout.height
        val muroInferior = ImageView(this)
        muroInferior.setBackgroundResource(R.drawable.asi_minijuego_2_muro)
        val altualturaMuro = (alturaPantalla *(0.2f + Random.nextFloat() * (0.4f))).toInt()

        // Configurar posición y tamaño de la tubería inferior
        val layoutParams = RelativeLayout.LayoutParams((anchuraPantalla*0.3f).toInt(), altualturaMuro)
        layoutParams.addRule(RelativeLayout.ALIGN_PARENT_RIGHT)
        muroInferior.layoutParams = layoutParams
        muroInferior.x = layout.width.toFloat()
        muroInferior.y = (alturaPantalla - altualturaMuro).toFloat()
        layout.addView(muroInferior)
        obstaculos.add(muroInferior)

        // Generar la tubería superior
        val muroSuperior = ImageView(this)
        muroSuperior.setBackgroundResource(R.drawable.asi_minijuego_2_muro)
        val alturaMuroSuperior = (alturaPantalla - altualturaMuro - alturaPantalla*0.2).toInt()
        muroSuperior.rotation = 180f
        // Configurar posición y tamaño de la tubería superior
        val layoutParamsSuperior = RelativeLayout.LayoutParams((anchuraPantalla*0.3f).toInt(), alturaMuroSuperior)
        layoutParamsSuperior.addRule(RelativeLayout.ALIGN_PARENT_RIGHT)
        muroSuperior.layoutParams = layoutParamsSuperior
        muroSuperior.x = layout.width.toFloat()
        muroSuperior.y = 0f
        layout.addView(muroSuperior)
        obstaculos.add(muroSuperior)
    }

    private fun moverObstaculos() {
        val obstaculosAEliminar = mutableListOf<ImageView>()

        for (obstaculo in obstaculos) {
            obstaculo.x -= velObs   // Mover obstáculo a la izquierda
            // Verificar si el obstáculo sale de la pantalla
            if (obstaculo.x + obstaculo.width < 0) {
                if(obstaculo.rotation ==180f){
                    puntuacion++
                }
                puntuacionTextView.text = "Puntuación: $puntuacion"
                obstaculosAEliminar.add(obstaculo) // Agregar a la lista de eliminación
            }
        }
        // Eliminar obstáculos que salieron de la pantalla
        for (obstaculo in obstaculosAEliminar) {
            layout.removeView(obstaculo)
            obstaculos.remove(obstaculo)
        }
    }
    private fun verificarColisiones() {
        for (obstaculo in obstaculos) {
            if (obstaculo.x < protagonista.x + protagonista.width - 120 &&
                obstaculo.x + obstaculo.width > protagonista.x + 80 &&
                obstaculo.y < protagonista.y + protagonista.height - 40 &&
                obstaculo.y + obstaculo.height > protagonista.y + 70
            ) {
                // Colisión detectada
                detenerJuego()
                break
            }
        }
    }

    private fun detenerJuego() {
        val flag = intent.getBooleanExtra("FLAG_KEY", false)
        jugando = false
        soundManager.stopSound()
        soundManager.playSound(R.raw.sound_over, 100)

        protagonista.clearAnimation()
        val animacionMuerte = AnimationUtils.loadAnimation(this, R.anim.caida)
        protagonista.startAnimation(animacionMuerte)
        for (obstaculo in obstaculos) {
            layout.removeView(obstaculo) // Eliminar cada obstáculo del layout
        }
        val handler = Handler(Looper.getMainLooper())
        handler.postDelayed({
            botonReinicio.visibility = View.VISIBLE
            val animacion = AnimationUtils.loadAnimation(this, R.anim.animatitu)
            botonReinicio.startAnimation(animacion)
        }, 1000)

        if(flag){
            botonReinicio.setOnClickListener {
                reiniciarJuego()
            }
        }else{
            // Devuelve la puntuación actual y finaliza la actividad
            val resultadoIntent = Intent()
            resultadoIntent.putExtra("PUNTUACION", puntuacion.toString())
            setResult(RESULT_OK, resultadoIntent)
            finish()
        }
    }

    private fun reiniciarJuego() {
        // Reiniciar variables del juego
        puntuacion = 0
        soundManager.playSound(R.raw.sound_button, 100)
        velObs = 10f
        puntuacionTextView.text = "Puntuación: $puntuacion"
        protagonista.y = (layout.height / 2).toFloat() // Reiniciar posición del protagonista
        botonReinicio.clearAnimation()
        botonReinicio.visibility = View.GONE // Ocultar botón de reinicio
        iniciarJuego() // Reiniciar el juego
    }

    private fun moverFondo() {
        // Mueve ambos fondos hacia la izquierda
        fondo1.x -= velocidadFondo
        fondo2.x -= velocidadFondo

        // Reinicia la posición cuando un fondo sale de la pantalla
        if (fondo1.x + fondo1.width * 4 < 0) {
            fondo1.x = fondo2.x + fondo2.width * 4
        }
        if (fondo2.x + fondo2.width * 4 < 0) {
            fondo2.x = fondo1.x + fondo1.width * 4
        }
    }
    override fun onPause() {
        super.onPause()
        soundManager.stopSound()
    }
    override fun onBackPressed() {
        val flag = intent.getBooleanExtra("FLAG_KEY", false)
        if (flag){
            super.onBackPressed()
        }
    }

}
class ModalFragment : DialogFragment() {

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflar el diseño del modal
        val view = inflater.inflate(R.layout.modal_minijuego2_tutorial, container, false)

        // Configurar el botón de cerrar
        val closeButton = view.findViewById<Button>(R.id.cerrar)
        closeButton.setOnClickListener {
            dismiss() // Cierra el modal
        }

        return view
    }
}
