package com.example.ellegadodepintia.minijuego3

import android.animation.Animator
import android.animation.ObjectAnimator
import android.content.Intent
import android.os.Bundle
import android.os.CountDownTimer
import android.os.Handler
import android.os.Looper
import android.widget.Button
import android.widget.GridLayout
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import android.animation.AnimatorListenerAdapter
import android.animation.ValueAnimator
import android.graphics.ColorMatrix
import android.graphics.ColorMatrixColorFilter
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.DialogFragment
import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.minijuego3.ModalFragment
import com.example.ellegadodepintia.soundManager.SoundManager

class Minijuego3 : AppCompatActivity() {
    private lateinit var portadaLayout: LinearLayout
    private lateinit var botonIniciar: ImageView
    private lateinit var gridLayout: GridLayout
    private lateinit var cronometroTextView: TextView
    private lateinit var botonVolver: ImageView;
    private lateinit var soundManager: SoundManager
    private var botonesCarta = mutableListOf<Button>()
    private var cartas = mutableListOf<Carta>()
    private var primieraCarta: Carta? = null
    private var segundaCarta: Carta? = null
    private var procesandose = false
    private var cuentraAtras: CountDownTimer? = null
    private var tiempoRestante: Long = 35000 // 1 minuto en milisegundos
    private var tiempoCorriendo = false

    override fun onCreate(savedInstanceState: Bundle?) {
        soundManager = SoundManager(this)

        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_minijuego3)
        inizializarVistas()
        setupBotonStart()

    }

    private fun inizializarVistas() {
        portadaLayout = findViewById(R.id.portadaLayout)
        botonIniciar = findViewById(R.id.btn_jugar)
        gridLayout = findViewById(R.id.gridLayout)
        cronometroTextView = findViewById(R.id.cronometro)
        botonVolver = findViewById(R.id.btn_volver)
        val tutorialButton = findViewById<ImageView>(R.id.descripcion)
        tutorialButton.setOnClickListener {
            // Mostrar el modal
            val modal = ModalFragment()
            modal.show(supportFragmentManager, "ModalFragment")
        }
    }

    private fun setupBotonStart() {
        botonIniciar.setOnClickListener {
            esconderPortada()
            soundManager.playSound(R.raw.sound_button,100)
            soundManager.playSoundLoop(R.raw.sound_memory,100)
            iniciarJuego()
        }
    }

    private fun esconderPortada() {
        botonVolver.visibility = ImageView.GONE
        portadaLayout.visibility = LinearLayout.GONE
        gridLayout.visibility = GridLayout.VISIBLE
        cronometroTextView.visibility = TextView.VISIBLE
    }

    private fun iniciarJuego() {
        inizcializarJuego()
        iniciarCronometro()
        voltearCartas()
    }

    private fun inizcializarJuego() {
        val images = listOf(
            R.drawable.asi_minijuego_3_vasito, R.drawable.asi_minijuego_3_copa_negra_brunida, R.drawable.asi_minijuego_3_jarra_de_pico, R.drawable.asi_minijuego_3_copa_tumba_75,
            R.drawable.asi_minijuego_3_sonajero_redondo, R.drawable.asi_minijuego_3_tres_en_raya, R.drawable.asi_minijuego_3_tapadera_zoomorfo, R.drawable.asi_minijuego_3_tintinnabulum
        )

        val imagenesCartas = (images + images).shuffled()
        // Obtén el ancho de la pantalla
        val displayMetrics = resources.displayMetrics
        val screenWidth = displayMetrics.widthPixels
        val numColumnas = 4.2
        val margin = screenWidth/35
        val tamannoCarta = (screenWidth - (margin * (numColumnas + 1))) / numColumnas
        for (i in 0 until 16) {
            val carta = Carta(id = i, imageResId = imagenesCartas[i])
            cartas.add(carta)
            val button = Button(this).apply {
                layoutParams = GridLayout.LayoutParams().apply {
                    width = tamannoCarta.toInt()
                    height = ((tamannoCarta + screenWidth*0.05).toInt())
                    setMargins(10, 10, 10, 10)
                }
                setBackgroundResource(R.drawable.asi_minijuego_3_trasero)
                setOnClickListener { cartaClickada(i) }
            }
            botonesCarta.add(button)
            gridLayout.addView(button)
        }
    }

    private fun iniciarCronometro() {
        cuentraAtras?.cancel()
        cuentraAtras = object : CountDownTimer(tiempoRestante, 1000) {
            override fun onTick(millisUntilFinished: Long) {
                tiempoRestante = millisUntilFinished
                actualizarContador()
            }
            override fun onFinish() {
                soundManager.stopSound()
                mostrarGameOver("¡Has perdido!")
            }
        }.start()
        tiempoCorriendo = true
    }

    private fun actualizarContador() {
        val segundos = (tiempoRestante / 1000).toInt()
        val minutos = segundos / 60
        val segundosRestantes = segundos % 60
        cronometroTextView.text = String.format("%02d:%02d", minutos, segundosRestantes)
    }

    private fun cartaClickada(position: Int) {
        if (procesandose) return
        soundManager.playSound(R.raw.sound_cardflip,100)
        val carta = cartas[position]
        val button = botonesCarta[position]
        if (carta.isFaceUp || carta.isMatched) return
        carta.isFaceUp = true
        if (primieraCarta == null) {
            primieraCarta = carta
            girarCarta(button, carta.imageResId)
        } else if (segundaCarta == null) {
            segundaCarta = carta
            girarCarta(button, carta.imageResId)
            procesandose = true
            comprobarMatch()
        }

    }

    private fun comprobarMatch() {
        Handler(Looper.getMainLooper()).postDelayed({
            if (primieraCarta?.imageResId == segundaCarta?.imageResId) {
                primieraCarta?.isMatched = true
                segundaCarta?.isMatched = true
                sumarTiempo()
                primieraCarta?.let { card ->
                    val button = botonesCarta[cartas.indexOf(card)]
                    iluminarCarta(button)
                }
                segundaCarta?.let { card ->
                    val button = botonesCarta[cartas.indexOf(card)]
                    iluminarCarta(button)}
                soundManager.playSound(R.raw.sound_match,60)
            } else {
                primieraCarta?.let { card ->
                    val button = botonesCarta[cartas.indexOf(card)]
                    card.isFaceUp = false
                    girarCarta(button, R.drawable.asi_minijuego_3_trasero, isReversing = true)
                }
                segundaCarta?.let { card ->
                    val button = botonesCarta[cartas.indexOf(card)]
                    card.isFaceUp = false
                    girarCarta(button, R.drawable.asi_minijuego_3_trasero, isReversing = true)

                }
                soundManager.playSound(R.raw.sound_cardflip,100)
            }
            primieraCarta = null
            segundaCarta = null
            procesandose = false
            comprobarFinJuego()

        }, 1000)
    }


    private fun comprobarFinJuego() {
        if (cartas.all { it.isMatched }) {
            soundManager.stopSound()
            tiempoCorriendo = false
            cuentraAtras?.cancel()
            mostrarGameOver("¡Has ganado!")
        }
    }

    private fun sumarTiempo() {
        tiempoRestante += 2000
        actualizarContador()
        iniciarCronometro()
    }

    private fun mostrarGameOver(mensaje: String) {
        Toast.makeText(this, mensaje, Toast.LENGTH_SHORT) .show()
        botonVolver.visibility = ImageView.VISIBLE
        val flag = intent.getBooleanExtra("FLAG_KEY", false)
        if(!flag){
            val resultadoIntent = Intent()
            resultadoIntent.putExtra("TIEMPO", tiempoRestante.toString())
            println(tiempoRestante)
            setResult(RESULT_OK, resultadoIntent)
            finish()
        }

        gridLayout.removeAllViews()
    }



    private fun girarCarta(button: Button, imageResId: Int, isReversing: Boolean = false) {
        val scale = applicationContext.resources.displayMetrics.density
        button.cameraDistance = 8000 * scale
        val flipOut = ObjectAnimator.ofFloat(button, "rotationY", 0f, 90f).apply {
            duration = 300
        }
        val flipIn = ObjectAnimator.ofFloat(button, "rotationY", 90f, 0f).apply {
            duration = 300
        }
        flipOut.addListener(object : AnimatorListenerAdapter() {
            override fun onAnimationEnd(animation: Animator) {
                button.setBackgroundResource(if (isReversing) R.drawable.asi_minijuego_3_trasero else imageResId)
                flipIn.start()
            }
        })
        flipOut.start()
    }

    private fun iluminarCarta(button: Button) {
        val matrix = ColorMatrix()
        matrix.setScale(1.5f, 1.5f, 1.5f, 1f)
        val colorFilter = ColorMatrixColorFilter(matrix)
        button.background.colorFilter = colorFilter
        val animator = ValueAnimator.ofFloat(1.5f, 1f)
        animator.duration = 1500
        animator.addUpdateListener { animation ->
            val value = animation.animatedValue as Float
            val scaleMatrix = ColorMatrix()
            scaleMatrix.setScale(value, value, value, 1f)
            button.background.colorFilter = ColorMatrixColorFilter(scaleMatrix)
        }
        animator.start()
    }

    private fun voltearCartas() {
        soundManager.playSound(R.raw.sound_cardflip,100)
        botonesCarta.forEachIndexed { index, button ->
            val card = cartas[index]
            card.isFaceUp = true
            girarCarta(button, card.imageResId)
        }
        Handler(Looper.getMainLooper()).postDelayed({
            soundManager.playSound(R.raw.sound_cardflip,100)
            botonesCarta.forEachIndexed { index, button ->
                val card = cartas[index]
                card.isFaceUp = false
                girarCarta(button, R.drawable.asi_minijuego_3_trasero, isReversing = true)

            }
        }, 2000)
    }
    override fun onPause() {
        super.onPause()
        soundManager.stopSound()
    }
    override fun onBackPressed() {
        val flag = intent.getBooleanExtra("FLAG_KEY", false)
        if (flag){
            cuentraAtras?.cancel()
            tiempoCorriendo = false
            super.onBackPressed()
        }
    }
}
class ModalFragment : DialogFragment() {

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflar el diseño del modal
        val view = inflater.inflate(R.layout.modal_minijuego3_tutorial, container, false)

        // Configurar el botón de cerrar
        val closeButton = view.findViewById<Button>(R.id.cerrar)
        closeButton.setOnClickListener {
            dismiss() // Cierra el modal
        }

        return view
    }
}
